@@ -42,9 +42,9 @@ module Data.Conduit.Blaze
   , reuseBufferStrategy
     ) where
 
-import Data.Conduit hiding (Source, Conduit, Sink, Pipe)
-import Control.Monad (unless)
-import Control.Monad.Trans.Class (lift)
+import Data.Conduit
+import Control.Monad (unless, liftM)
+import Control.Monad.Trans.Class (lift, MonadTrans)
 
 import qualified Data.ByteString                   as S
 
@@ -54,14 +54,14 @@ import Blaze.ByteString.Builder.Internal.Buffer
 
 -- | Incrementally execute builders and pass on the filled chunks as
 -- bytestrings.
-builderToByteString :: MonadUnsafeIO m => GInfConduit Builder m S.ByteString
+builderToByteString :: MonadUnsafeIO m => Conduit Builder m S.ByteString
 builderToByteString =
   builderToByteStringWith (allNewBuffersStrategy defaultBufferSize)
 
 -- |
 --
 -- Since 0.0.2
-builderToByteStringFlush :: MonadUnsafeIO m => GInfConduit (Flush Builder) m (Flush S.ByteString)
+builderToByteStringFlush :: MonadUnsafeIO m => Conduit (Flush Builder) m (Flush S.ByteString)
 builderToByteStringFlush =
   builderToByteStringWithFlush (allNewBuffersStrategy defaultBufferSize)
 
@@ -74,7 +74,7 @@ builderToByteStringFlush =
 -- as control is returned from the inner sink!
 unsafeBuilderToByteString :: MonadUnsafeIO m
                           => IO Buffer  -- action yielding the inital buffer.
-                          -> GInfConduit Builder m S.ByteString
+                          -> Conduit Builder m S.ByteString
 unsafeBuilderToByteString = builderToByteStringWith . reuseBufferStrategy
 
 
@@ -84,12 +84,12 @@ unsafeBuilderToByteString = builderToByteStringWith . reuseBufferStrategy
 -- INV: All bytestrings passed to the inner sink are non-empty.
 builderToByteStringWith :: MonadUnsafeIO m
                         => BufferAllocStrategy
-                        -> GInfConduit Builder m S.ByteString
+                        -> Conduit Builder m S.ByteString
 builderToByteStringWith =
-    mapOutputMaybe unChunk . mapInput Chunk unChunk . builderToByteStringWithFlush
+    helper (liftM (fmap Chunk) await) yield'
   where
-    unChunk Flush = Nothing
-    unChunk (Chunk y) = Just y
+    yield' Flush = return ()
+    yield' (Chunk bs) = yield bs
 
 -- |
 --
@@ -97,20 +97,26 @@ builderToByteStringWith =
 builderToByteStringWithFlush
     :: MonadUnsafeIO m
     => BufferAllocStrategy
-    -> GInfConduit (Flush Builder) m (Flush S.ByteString)
-builderToByteStringWithFlush (ioBufInit, nextBuf) =
+    -> Conduit (Flush Builder) m (Flush S.ByteString)
+builderToByteStringWithFlush = helper await yield
+
+helper :: (MonadUnsafeIO m, Monad (t m), MonadTrans t)
+       => t m (Maybe (Flush Builder))
+       -> (Flush S.ByteString -> t m ())
+       -> BufferAllocStrategy
+       -> t m ()
+helper await' yield' (ioBufInit, nextBuf) =
     loop ioBufInit
   where
     loop ioBuf = do
-        awaitE >>= either (close ioBuf) (cont' ioBuf)
+        await' >>= maybe (close ioBuf) (cont' ioBuf)
 
-    cont' ioBuf Flush = push ioBuf flush $ \ioBuf' -> yield Flush >> loop ioBuf'
+    cont' ioBuf Flush = push ioBuf flush $ \ioBuf' -> yield' Flush >> loop ioBuf'
     cont' ioBuf (Chunk builder) = push ioBuf builder loop
 
-    close ioBuf r = do
+    close ioBuf = do
         buf <- lift $ unsafeLiftIO $ ioBuf
-        maybe (return ()) (yield . Chunk) (unsafeFreezeNonEmptyBuffer buf)
-        return r
+        maybe (return ()) (yield' . Chunk) (unsafeFreezeNonEmptyBuffer buf)
 
     push ioBuf0 x continue = do
         go (unBuilder x (buildStep finalStep)) ioBuf0
@@ -133,12 +139,12 @@ builderToByteStringWithFlush (ioBufInit, nextBuf) =
                             go bStep' ioBuf'
                     case unsafeFreezeNonEmptyBuffer buf' of
                         Nothing -> return ()
-                        Just bs -> yield (Chunk bs)
+                        Just bs -> yield' (Chunk bs)
                     cont
                 InsertByteString op' bs bStep' -> do
                     let buf' = updateEndOfSlice buf op'
                     case unsafeFreezeNonEmptyBuffer buf' of
                         Nothing -> return ()
-                        Just bs' -> yield $ Chunk bs'
-                    unless (S.null bs) $ yield $ Chunk bs
+                        Just bs' -> yield' $ Chunk bs'
+                    unless (S.null bs) $ yield' $ Chunk bs
                     lift (unsafeLiftIO $ nextBuf 1 buf') >>= go bStep'
